import 'package:flutter/widgets.dart';

import 'package:responsive_layout_builder/src/layout_size.dart';
import 'package:responsive_layout_builder/src/screen_size.dart';
import 'package:responsive_layout_builder/src/screen_size_settings.dart';

/// Gets type of mobile layout size
///
/// [width] argument must not be null
MobileLayoutSize getMobileLayoutSize({
  @required double width,
  ScreenSizeSettings sizes = const ScreenSizeSettings.portrait(),
}) {
  MobileLayoutSize result;

  if (width > sizes.watch && width <= sizes.smallMobile) {
    result = MobileLayoutSize.small;
  } else if (width > sizes.smallMobile && width <= sizes.mediumMobile) {
    result = MobileLayoutSize.medium;
  } else {
    result = MobileLayoutSize.large;
  }

  return result;
}

/// Gets type of tablet layout size
///
/// [width] argument must not be null
TabletLayoutSize getTabletLayoutSize({
  @required double width,
  ScreenSizeSettings sizes = const ScreenSizeSettings.portrait(),
}) {
  TabletLayoutSize result;

  if (width > sizes.largeMobile && width <= sizes.smallTablet) {
    result = TabletLayoutSize.small;
  } else {
    result = TabletLayoutSize.large;
  }

  return result;
}

/// Gets type of screen size based on width
///
/// [width] argument must not be null
ScreenSize getScreenSize({
  @required double width,
  LayoutSize defaultSize = LayoutSize.mobile,
  Orientation orientation = Orientation.portrait,
  ScreenSizeSettings sizes = const ScreenSizeSettings.portrait(),
}) {
  LayoutSize size;
  MobileLayoutSize mobile;
  TabletLayoutSize tablet;

  if (width <= sizes.watch) {
    size = LayoutSize.watch;
  } else if (width > sizes.watch && width <= sizes.largeMobile) {
    size = LayoutSize.mobile;
    mobile = getMobileLayoutSize(sizes: sizes, width: width);
  } else if (width > sizes.largeMobile && width <= sizes.largeTablet) {
    size = LayoutSize.tablet;
    tablet = getTabletLayoutSize(sizes: sizes, width: width);
  } else if (width > sizes.largeTablet) {
    size = LayoutSize.desktop;
  } else {
    size = defaultSize ?? LayoutSize.mobile;
  }

  return ScreenSize(
    size: size,
    mobile: mobile,
    tablet: tablet,
    orientation: orientation,
  );
}

/// Gets type of screen size based on [BuildContext]
///
/// [context] argument must not be null
ScreenSize getContextualScreenSize({
  @required BuildContext context,
  LayoutSize defaultSize = LayoutSize.mobile,
  Orientation orientation = Orientation.portrait,
  ScreenSizeSettings sizes = const ScreenSizeSettings.portrait(),
}) =>
    getScreenSize(
      width: MediaQuery.of(context).size.width,
      defaultSize: defaultSize,
      orientation: orientation,
      sizes: sizes,
    );
